# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.documents_single_get_response_data_attributes_home_page_content import (
        DocumentsSingleGetResponseDataAttributesHomePageContent,
    )
    from ..models.documents_single_get_response_data_attributes_outline_numbering import (
        DocumentsSingleGetResponseDataAttributesOutlineNumbering,
    )
    from ..models.documents_single_get_response_data_attributes_rendering_layouts_item import (
        DocumentsSingleGetResponseDataAttributesRenderingLayoutsItem,
    )


T = TypeVar("T", bound="DocumentsSingleGetResponseDataAttributes")


@_attrs_define
class DocumentsSingleGetResponseDataAttributes:
    """
    Attributes:
        auto_suspect (Union[Unset, bool]):
        branched_with_initialized_fields (Union[Unset, List[str]]):
        branched_with_query (Union[Unset, str]):  Example: Branched with Query.
        created (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        derived_fields (Union[Unset, List[str]]):
        derived_from_link_role (Union[Unset, str]):  Example: relates_to.
        home_page_content (Union[Unset, DocumentsSingleGetResponseDataAttributesHomePageContent]):
        module_folder (Union[Unset, str]):  Example: MySpaceId.
        module_name (Union[Unset, str]):  Example: MyDocumentId.
        outline_numbering (Union[Unset, DocumentsSingleGetResponseDataAttributesOutlineNumbering]):
        rendering_layouts (Union[Unset, List['DocumentsSingleGetResponseDataAttributesRenderingLayoutsItem']]):
        status (Union[Unset, str]):  Example: draft.
        structure_link_role (Union[Unset, str]):  Example: relates_to.
        title (Union[Unset, str]):  Example: Title.
        type (Union[Unset, str]):  Example: req_specification.
        updated (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        uses_outline_numbering (Union[Unset, bool]):
    """

    auto_suspect: Union[Unset, bool] = UNSET
    branched_with_initialized_fields: Union[Unset, List[str]] = UNSET
    branched_with_query: Union[Unset, str] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    derived_fields: Union[Unset, List[str]] = UNSET
    derived_from_link_role: Union[Unset, str] = UNSET
    home_page_content: Union[
        Unset, "DocumentsSingleGetResponseDataAttributesHomePageContent"
    ] = UNSET
    module_folder: Union[Unset, str] = UNSET
    module_name: Union[Unset, str] = UNSET
    outline_numbering: Union[
        Unset, "DocumentsSingleGetResponseDataAttributesOutlineNumbering"
    ] = UNSET
    rendering_layouts: Union[
        Unset,
        List["DocumentsSingleGetResponseDataAttributesRenderingLayoutsItem"],
    ] = UNSET
    status: Union[Unset, str] = UNSET
    structure_link_role: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    uses_outline_numbering: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        auto_suspect = self.auto_suspect

        branched_with_initialized_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.branched_with_initialized_fields, Unset):
            branched_with_initialized_fields = (
                self.branched_with_initialized_fields
            )

        branched_with_query = self.branched_with_query

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        derived_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.derived_fields, Unset):
            derived_fields = self.derived_fields

        derived_from_link_role = self.derived_from_link_role

        home_page_content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.home_page_content, Unset):
            home_page_content = self.home_page_content.to_dict()

        module_folder = self.module_folder

        module_name = self.module_name

        outline_numbering: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.outline_numbering, Unset):
            outline_numbering = self.outline_numbering.to_dict()

        rendering_layouts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rendering_layouts, Unset):
            rendering_layouts = []
            for rendering_layouts_item_data in self.rendering_layouts:
                rendering_layouts_item = rendering_layouts_item_data.to_dict()
                rendering_layouts.append(rendering_layouts_item)

        status = self.status

        structure_link_role = self.structure_link_role

        title = self.title

        type = self.type

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        uses_outline_numbering = self.uses_outline_numbering

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if auto_suspect is not UNSET:
            field_dict["autoSuspect"] = auto_suspect
        if branched_with_initialized_fields is not UNSET:
            field_dict["branchedWithInitializedFields"] = (
                branched_with_initialized_fields
            )
        if branched_with_query is not UNSET:
            field_dict["branchedWithQuery"] = branched_with_query
        if created is not UNSET:
            field_dict["created"] = created
        if derived_fields is not UNSET:
            field_dict["derivedFields"] = derived_fields
        if derived_from_link_role is not UNSET:
            field_dict["derivedFromLinkRole"] = derived_from_link_role
        if home_page_content is not UNSET:
            field_dict["homePageContent"] = home_page_content
        if module_folder is not UNSET:
            field_dict["moduleFolder"] = module_folder
        if module_name is not UNSET:
            field_dict["moduleName"] = module_name
        if outline_numbering is not UNSET:
            field_dict["outlineNumbering"] = outline_numbering
        if rendering_layouts is not UNSET:
            field_dict["renderingLayouts"] = rendering_layouts
        if status is not UNSET:
            field_dict["status"] = status
        if structure_link_role is not UNSET:
            field_dict["structureLinkRole"] = structure_link_role
        if title is not UNSET:
            field_dict["title"] = title
        if type is not UNSET:
            field_dict["type"] = type
        if updated is not UNSET:
            field_dict["updated"] = updated
        if uses_outline_numbering is not UNSET:
            field_dict["usesOutlineNumbering"] = uses_outline_numbering

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.documents_single_get_response_data_attributes_home_page_content import (
            DocumentsSingleGetResponseDataAttributesHomePageContent,
        )
        from ..models.documents_single_get_response_data_attributes_outline_numbering import (
            DocumentsSingleGetResponseDataAttributesOutlineNumbering,
        )
        from ..models.documents_single_get_response_data_attributes_rendering_layouts_item import (
            DocumentsSingleGetResponseDataAttributesRenderingLayoutsItem,
        )

        d = src_dict.copy()
        auto_suspect = d.pop("autoSuspect", UNSET)

        branched_with_initialized_fields = cast(
            List[str], d.pop("branchedWithInitializedFields", UNSET)
        )

        branched_with_query = d.pop("branchedWithQuery", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        derived_fields = cast(List[str], d.pop("derivedFields", UNSET))

        derived_from_link_role = d.pop("derivedFromLinkRole", UNSET)

        _home_page_content = d.pop("homePageContent", UNSET)
        home_page_content: Union[
            Unset, DocumentsSingleGetResponseDataAttributesHomePageContent
        ]
        if isinstance(_home_page_content, Unset):
            home_page_content = UNSET
        else:
            home_page_content = DocumentsSingleGetResponseDataAttributesHomePageContent.from_dict(
                _home_page_content
            )

        module_folder = d.pop("moduleFolder", UNSET)

        module_name = d.pop("moduleName", UNSET)

        _outline_numbering = d.pop("outlineNumbering", UNSET)
        outline_numbering: Union[
            Unset, DocumentsSingleGetResponseDataAttributesOutlineNumbering
        ]
        if isinstance(_outline_numbering, Unset):
            outline_numbering = UNSET
        else:
            outline_numbering = DocumentsSingleGetResponseDataAttributesOutlineNumbering.from_dict(
                _outline_numbering
            )

        rendering_layouts = []
        _rendering_layouts = d.pop("renderingLayouts", UNSET)
        for rendering_layouts_item_data in _rendering_layouts or []:
            rendering_layouts_item = DocumentsSingleGetResponseDataAttributesRenderingLayoutsItem.from_dict(
                rendering_layouts_item_data
            )

            rendering_layouts.append(rendering_layouts_item)

        status = d.pop("status", UNSET)

        structure_link_role = d.pop("structureLinkRole", UNSET)

        title = d.pop("title", UNSET)

        type = d.pop("type", UNSET)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset):
            updated = UNSET
        else:
            updated = isoparse(_updated)

        uses_outline_numbering = d.pop("usesOutlineNumbering", UNSET)

        documents_single_get_response_data_attributes_obj = cls(
            auto_suspect=auto_suspect,
            branched_with_initialized_fields=branched_with_initialized_fields,
            branched_with_query=branched_with_query,
            created=created,
            derived_fields=derived_fields,
            derived_from_link_role=derived_from_link_role,
            home_page_content=home_page_content,
            module_folder=module_folder,
            module_name=module_name,
            outline_numbering=outline_numbering,
            rendering_layouts=rendering_layouts,
            status=status,
            structure_link_role=structure_link_role,
            title=title,
            type=type,
            updated=updated,
            uses_outline_numbering=uses_outline_numbering,
        )

        documents_single_get_response_data_attributes_obj.additional_properties = (
            d
        )
        return documents_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import json

import pytest
import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_WIL_CREATED_RESPONSE,
    TEST_WIL_DELETE2_REQUEST,
    TEST_WIL_DELETE_REQUEST,
    TEST_WIL_DELETED_REQUEST,
    TEST_WIL_MULTI_POST_REQUEST,
    TEST_WIL_NEXT_PAGE_RESPONSE,
    TEST_WIL_NO_NEXT_PAGE_RESPONSE,
    TEST_WIL_POSTED_REQUEST,
)


def test_get_work_item_links_single_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(
        TEST_WIL_NO_NEXT_PAGE_RESPONSE,
        encoding="utf8",
    ) as f:
        httpx_mock.add_response(json=json.load(f))

    work_item_links = client.work_items.links.get_all(
        "MyWorkItemId",
        include="workitem",
        fields={"fields[linkedworkitems]": "id,role"},
    )

    query = {
        "fields[linkedworkitems]": "id,role",
        "page[size]": "100",
        "page[number]": "1",
        "include": "workitem",
    }
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    assert len(work_item_links) == 1
    assert len(reqs) == 1
    assert work_item_links[0] == polarion_api.WorkItemLink(
        "MyWorkItemId", "MyWorkItemId2", "relates_to", True, "MyProjectId"
    )


def test_get_work_item_links_multi_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(
        TEST_WIL_NEXT_PAGE_RESPONSE,
        encoding="utf8",
    ) as f:
        httpx_mock.add_response(json=json.load(f))
    with open(
        TEST_WIL_NO_NEXT_PAGE_RESPONSE,
        encoding="utf8",
    ) as f:
        httpx_mock.add_response(json=json.load(f))

    work_items = client.work_items.links.get_all("MyWorkItemId")
    query = {
        "fields[linkedworkitems]": "id,role,suspect",
        "page[size]": "100",
        "page[number]": "1",
    }
    reqs = httpx_mock.get_requests()

    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    assert reqs[1].method == "GET"
    query["page[number]"] = "2"
    assert dict(reqs[1].url.params) == query
    assert len(work_items) == 2
    assert len(reqs) == 2
    assert work_items[0].suspect is False


def test_delete_work_item_link(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.links.delete(
        polarion_api.WorkItemLink(
            "MyWorkItemId", "MyWorkItemId2", "parent", True, "MyProjectId"
        )
    )

    req = httpx_mock.get_request()

    assert req is not None and req.method == "DELETE"
    with open(TEST_WIL_DELETED_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_delete_work_item_links(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.links.delete(
        [
            polarion_api.WorkItemLink(
                "MyWorkItemId", "MyWorkItemId2", "parent", True, "MyProjectId"
            ),
            polarion_api.WorkItemLink(
                "MyWorkItemId", "MyWorkItemId3", "parent", True
            ),
        ]
    )

    req = httpx_mock.get_request()

    assert req is not None and req.method == "DELETE"
    with open(TEST_WIL_DELETE_REQUEST, encoding="utf8") as f:
        expected_request = json.load(f)

    assert json.loads(req.content.decode()) == expected_request


def test_delete_work_item_links_multi_primary(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)
    httpx_mock.add_response(204)

    client.work_items.links.delete(
        [
            polarion_api.WorkItemLink(
                "MyWorkItemId", "MyWorkItemId2", "parent", True, "MyProjectId"
            ),
            polarion_api.WorkItemLink(
                "MyWorkItemId2", "MyWorkItemId3", "parent", True
            ),
        ]
    )

    reqs = httpx_mock.get_requests()
    with open(TEST_WIL_DELETED_REQUEST, encoding="utf8") as f:
        expected_request = json.load(f)

    with open(TEST_WIL_DELETE2_REQUEST, encoding="utf8") as f:
        expected_request2 = json.load(f)

    assert len(reqs) == 2
    assert reqs[0].method == "DELETE"
    assert reqs[1].method == "DELETE"
    assert json.loads(reqs[0].content.decode()) == expected_request
    assert json.loads(reqs[1].content.decode()) == expected_request2


def test_create_work_item_link(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WIL_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))

    client.work_items.links.create(
        polarion_api.WorkItemLink(
            "MyWorkItemId", "MyWorkItemId2", "relates_to", True
        )
    )

    req = httpx_mock.get_request()

    assert req is not None and req.method == "POST"
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId/linkedworkitems")
    with open(TEST_WIL_POSTED_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected


def test_create_work_item_links_different_primaries(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WIL_CREATED_RESPONSE, encoding="utf8") as f:
        content = json.load(f)

    httpx_mock.add_response(201, json=content)
    httpx_mock.add_response(201, json=content)

    client.work_items.links.create(
        [
            polarion_api.WorkItemLink(
                "MyWorkItemId", "MyWorkItemId2", "relates_to", True
            ),
            polarion_api.WorkItemLink(
                "MyWorkItemId3", "MyWorkItemId2", "relates_to", True
            ),
        ]
    )

    reqs = httpx_mock.get_requests()

    assert len(reqs) == 2
    assert reqs[0].method == "POST"
    assert reqs[1].method == "POST"

    assert reqs[0].url.path.endswith(
        "PROJ/workitems/MyWorkItemId/linkedworkitems"
    )
    assert reqs[1].url.path.endswith(
        "PROJ/workitems/MyWorkItemId3/linkedworkitems"
    )

    with open(TEST_WIL_POSTED_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(reqs[0].content.decode()) == expected
    assert json.loads(reqs[1].content.decode()) == expected


def test_create_work_item_links_same_primaries(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WIL_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))

    client.work_items.links.create(
        [
            polarion_api.WorkItemLink(
                "MyWorkItemId",
                "MyWorkItemId2",
                "relates_to",
                True,
                "MyProjectId",
            ),
            polarion_api.WorkItemLink(
                "MyWorkItemId", "MyWorkItemId3", "parent", False
            ),
        ]
    )

    req = httpx_mock.get_request()

    assert req is not None and req.method == "POST"
    with open(TEST_WIL_MULTI_POST_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected


def test_get_work_item_links_error_first_request(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    caplog: pytest.LogCaptureFixture,
):
    httpx_mock.add_response(502, content=b"Test")
    with open(
        TEST_WIL_NO_NEXT_PAGE_RESPONSE,
        encoding="utf8",
    ) as f:
        httpx_mock.add_response(json=json.load(f))

    work_item_links = client.work_items.links.get_all(
        "MyWorkItemId",
    )

    reqs = httpx_mock.get_requests()
    assert len(caplog.record_tuples) == 1
    _, level, message = caplog.record_tuples[0]
    assert level == 30
    assert (
        message == "Will retry after failing on first attempt, due to "
        "the following error "
        "(<HTTPStatus.BAD_GATEWAY: 502>, b'Test')"
    )
    assert len(reqs) == 2
    assert work_item_links[0] == polarion_api.WorkItemLink(
        "MyWorkItemId", "MyWorkItemId2", "relates_to", True, "MyProjectId"
    )



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MoveWorkItemToDocumentRequestBody")


@_attrs_define
class MoveWorkItemToDocumentRequestBody:
    """
    Attributes:
        next_part (Union[Unset, str]):  Example: MyProjectId/MySpaceId/MyDocumentId/workitem_MyWorkItemId.
        previous_part (Union[Unset, str]):  Example: MyProjectId/MySpaceId/MyDocumentId/workitem_MyWorkItemId.
        target_document (Union[Unset, str]):  Example: MyProjectId/MySpaceId/MyDocumentId.
    """

    next_part: Union[Unset, str] = UNSET
    previous_part: Union[Unset, str] = UNSET
    target_document: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        next_part = self.next_part

        previous_part = self.previous_part

        target_document = self.target_document

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if next_part is not UNSET:
            field_dict["nextPart"] = next_part
        if previous_part is not UNSET:
            field_dict["previousPart"] = previous_part
        if target_document is not UNSET:
            field_dict["targetDocument"] = target_document

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        next_part = d.pop("nextPart", UNSET)

        previous_part = d.pop("previousPart", UNSET)

        target_document = d.pop("targetDocument", UNSET)

        move_work_item_to_document_request_body_obj = cls(
            next_part=next_part,
            previous_part=previous_part,
            target_document=target_document,
        )

        move_work_item_to_document_request_body_obj.additional_properties = d
        return move_work_item_to_document_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.users_list_post_request_data_item_relationships_global_roles import (
        UsersListPostRequestDataItemRelationshipsGlobalRoles,
    )
    from ..models.users_list_post_request_data_item_relationships_project_roles import (
        UsersListPostRequestDataItemRelationshipsProjectRoles,
    )
    from ..models.users_list_post_request_data_item_relationships_user_groups import (
        UsersListPostRequestDataItemRelationshipsUserGroups,
    )


T = TypeVar("T", bound="UsersListPostRequestDataItemRelationships")


@_attrs_define
class UsersListPostRequestDataItemRelationships:
    """
    Attributes:
        global_roles (Union[Unset, UsersListPostRequestDataItemRelationshipsGlobalRoles]):
        project_roles (Union[Unset, UsersListPostRequestDataItemRelationshipsProjectRoles]):
        user_groups (Union[Unset, UsersListPostRequestDataItemRelationshipsUserGroups]):
    """

    global_roles: Union[
        Unset, "UsersListPostRequestDataItemRelationshipsGlobalRoles"
    ] = UNSET
    project_roles: Union[
        Unset, "UsersListPostRequestDataItemRelationshipsProjectRoles"
    ] = UNSET
    user_groups: Union[
        Unset, "UsersListPostRequestDataItemRelationshipsUserGroups"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        global_roles: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.global_roles, Unset):
            global_roles = self.global_roles.to_dict()

        project_roles: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project_roles, Unset):
            project_roles = self.project_roles.to_dict()

        user_groups: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user_groups, Unset):
            user_groups = self.user_groups.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if global_roles is not UNSET:
            field_dict["globalRoles"] = global_roles
        if project_roles is not UNSET:
            field_dict["projectRoles"] = project_roles
        if user_groups is not UNSET:
            field_dict["userGroups"] = user_groups

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.users_list_post_request_data_item_relationships_global_roles import (
            UsersListPostRequestDataItemRelationshipsGlobalRoles,
        )
        from ..models.users_list_post_request_data_item_relationships_project_roles import (
            UsersListPostRequestDataItemRelationshipsProjectRoles,
        )
        from ..models.users_list_post_request_data_item_relationships_user_groups import (
            UsersListPostRequestDataItemRelationshipsUserGroups,
        )

        d = src_dict.copy()
        _global_roles = d.pop("globalRoles", UNSET)
        global_roles: Union[
            Unset, UsersListPostRequestDataItemRelationshipsGlobalRoles
        ]
        if isinstance(_global_roles, Unset):
            global_roles = UNSET
        else:
            global_roles = (
                UsersListPostRequestDataItemRelationshipsGlobalRoles.from_dict(
                    _global_roles
                )
            )

        _project_roles = d.pop("projectRoles", UNSET)
        project_roles: Union[
            Unset, UsersListPostRequestDataItemRelationshipsProjectRoles
        ]
        if isinstance(_project_roles, Unset):
            project_roles = UNSET
        else:
            project_roles = UsersListPostRequestDataItemRelationshipsProjectRoles.from_dict(
                _project_roles
            )

        _user_groups = d.pop("userGroups", UNSET)
        user_groups: Union[
            Unset, UsersListPostRequestDataItemRelationshipsUserGroups
        ]
        if isinstance(_user_groups, Unset):
            user_groups = UNSET
        else:
            user_groups = (
                UsersListPostRequestDataItemRelationshipsUserGroups.from_dict(
                    _user_groups
                )
            )

        users_list_post_request_data_item_relationships_obj = cls(
            global_roles=global_roles,
            project_roles=project_roles,
            user_groups=user_groups,
        )

        users_list_post_request_data_item_relationships_obj.additional_properties = (
            d
        )
        return users_list_post_request_data_item_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class PlansListPostRequestDataItemAttributesCalculationType(str, Enum):
    CUSTOMFIELDBASED = "customFieldBased"
    TIMEBASED = "timeBased"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
import json

import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_TREC_CREATED_RESPONSE,
    TEST_TREC_NEXT_RESPONSE,
    TEST_TREC_NO_NEXT_RESPONSE,
    TEST_TREC_PATCH_REQUEST,
    TEST_TREC_POST_REQUEST,
)


def test_get_test_records_multi_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_TREC_NEXT_RESPONSE, encoding="utf8") as f:
        content = json.load(f)
        httpx_mock.add_response(json=content)
        httpx_mock.add_response(json=content)

    with open(TEST_TREC_NO_NEXT_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    test_records = client.test_runs.records.get_all(
        "123", fields={"test_records": "@all"}
    )

    query = {
        "page[size]": "100",
        "page[number]": "1",
        "fields[test_records]": "@all",
    }
    reqs = httpx_mock.get_requests()

    assert len(reqs) == 3
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    query["page[number]"] = "2"
    assert dict(reqs[1].url.params) == query
    query["page[number]"] = "3"
    assert dict(reqs[2].url.params) == query
    assert len(test_records) == 3
    assert test_records[0].result == "passed"
    assert test_records[0].iteration == 0
    assert test_records[0].duration == 0
    assert test_records[0].comment
    assert test_records[0].comment.value == "My text value"
    assert test_records[0].comment.type == "text/html"
    assert test_records[0].work_item_id == "MyTestcaseId2"
    assert test_records[0].work_item_revision == "1234"


def test_create_test_records(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_TREC_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))

    test_run_id = "asdfg"

    tr_1 = polarion_api.TestRecord(
        test_run_id,
        "MyProjectId",
        "MyWorkItemId",
        "0",
        executed=datetime.datetime.utcfromtimestamp(0),
        duration=0,
        result="passed",
        comment=polarion_api.TextContent("text/html", "My text value"),
    )
    tr_2 = polarion_api.TestRecord(
        test_run_id,
        "MyProjectId",
        "MyWorkItemId",
        "1234",
        executed=datetime.datetime.utcfromtimestamp(0),
        duration=1,
        result="failed",
        comment=polarion_api.TextContent("text/html", "My text value 2"),
    )

    client.test_runs.records.create([tr_1, tr_2])

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 1
    req_data = json.loads(reqs[0].content.decode("utf-8"))
    with open(TEST_TREC_POST_REQUEST, encoding="utf8") as f:
        expected_req = json.load(f)

    assert req_data == expected_req
    assert reqs[0].url.path.endswith(f"/testruns/{test_run_id}/testrecords")
    assert tr_1.iteration == 0
    assert tr_2.iteration == 1


def test_update_test_record(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    test_run_id = "asdfg"
    work_item_id = "MyWorkItemId"
    work_item_project = "MyProjectId"

    tr_1 = polarion_api.TestRecord(
        test_run_id,
        work_item_project,
        work_item_id,
        iteration=4,
        duration=1337.5,
        result="passed",
        comment=polarion_api.TextContent("text/html", "My text value"),
    )

    client.test_runs.records.update(tr_1)

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 1
    req_data = json.loads(reqs[0].content.decode("utf-8"))
    with open(TEST_TREC_PATCH_REQUEST, encoding="utf8") as f:
        expected_req = json.load(f)

    assert req_data == expected_req
    assert reqs[0].url.path.endswith(
        f"/testruns/{test_run_id}"
        f"/testrecords/{work_item_project}/{work_item_id}/4"
    )



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.teststep_results_list_get_response_data_item_meta_errors_item_source_resource import (
        TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource,
    )


T = TypeVar(
    "T", bound="TeststepResultsListGetResponseDataItemMetaErrorsItemSource"
)


@_attrs_define
class TeststepResultsListGetResponseDataItemMetaErrorsItemSource:
    """
    Attributes:
        parameter (Union[Unset, str]): String indicating which URI query parameter caused the error. Example: revision.
        pointer (Union[Unset, str]): JSON Pointer to the associated entity in the request document. Example: $.data.
        resource (Union[Unset, TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource]): Resource causing
            the error.
    """

    parameter: Union[Unset, str] = UNSET
    pointer: Union[Unset, str] = UNSET
    resource: Union[
        Unset,
        "TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource",
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        parameter = self.parameter

        pointer = self.pointer

        resource: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.resource, Unset):
            resource = self.resource.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if parameter is not UNSET:
            field_dict["parameter"] = parameter
        if pointer is not UNSET:
            field_dict["pointer"] = pointer
        if resource is not UNSET:
            field_dict["resource"] = resource

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.teststep_results_list_get_response_data_item_meta_errors_item_source_resource import (
            TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource,
        )

        d = src_dict.copy()
        parameter = d.pop("parameter", UNSET)

        pointer = d.pop("pointer", UNSET)

        _resource = d.pop("resource", UNSET)
        resource: Union[
            Unset,
            TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource,
        ]
        if isinstance(_resource, Unset):
            resource = UNSET
        else:
            resource = TeststepResultsListGetResponseDataItemMetaErrorsItemSourceResource.from_dict(
                _resource
            )

        teststep_results_list_get_response_data_item_meta_errors_item_source_obj = cls(
            parameter=parameter,
            pointer=pointer,
            resource=resource,
        )

        teststep_results_list_get_response_data_item_meta_errors_item_source_obj.additional_properties = (
            d
        )
        return teststep_results_list_get_response_data_item_meta_errors_item_source_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BranchDocumentRequestBody")


@_attrs_define
class BranchDocumentRequestBody:
    """
    Attributes:
        copy_workflow_status_and_signatures (Union[Unset, bool]): Specifies that workflow status and signatures should
            be copied to the branched document.
        query (Union[Unset, str]): Specifies optional filtering query. Example: status:open.
        target_document_name (Union[Unset, str]): Name for new Document. Example: MyDocumentId.
        target_project_id (Union[Unset, str]): Project where new document will be created. Example: MyProjectId.
        target_space_id (Union[Unset, str]): Space where new document will be created. Example: MySpaceId.
    """

    copy_workflow_status_and_signatures: Union[Unset, bool] = UNSET
    query: Union[Unset, str] = UNSET
    target_document_name: Union[Unset, str] = UNSET
    target_project_id: Union[Unset, str] = UNSET
    target_space_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        copy_workflow_status_and_signatures = (
            self.copy_workflow_status_and_signatures
        )

        query = self.query

        target_document_name = self.target_document_name

        target_project_id = self.target_project_id

        target_space_id = self.target_space_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if copy_workflow_status_and_signatures is not UNSET:
            field_dict["copyWorkflowStatusAndSignatures"] = (
                copy_workflow_status_and_signatures
            )
        if query is not UNSET:
            field_dict["query"] = query
        if target_document_name is not UNSET:
            field_dict["targetDocumentName"] = target_document_name
        if target_project_id is not UNSET:
            field_dict["targetProjectId"] = target_project_id
        if target_space_id is not UNSET:
            field_dict["targetSpaceId"] = target_space_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        copy_workflow_status_and_signatures = d.pop(
            "copyWorkflowStatusAndSignatures", UNSET
        )

        query = d.pop("query", UNSET)

        target_document_name = d.pop("targetDocumentName", UNSET)

        target_project_id = d.pop("targetProjectId", UNSET)

        target_space_id = d.pop("targetSpaceId", UNSET)

        branch_document_request_body_obj = cls(
            copy_workflow_status_and_signatures=copy_workflow_status_and_signatures,
            query=query,
            target_document_name=target_document_name,
            target_project_id=target_project_id,
            target_space_id=target_space_id,
        )

        branch_document_request_body_obj.additional_properties = d
        return branch_document_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitems_list_get_response_data_item_relationships_approvals_data_item import (
        WorkitemsListGetResponseDataItemRelationshipsApprovalsDataItem,
    )
    from ..models.workitems_list_get_response_data_item_relationships_approvals_meta import (
        WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta,
    )


T = TypeVar(
    "T", bound="WorkitemsListGetResponseDataItemRelationshipsApprovals"
)


@_attrs_define
class WorkitemsListGetResponseDataItemRelationshipsApprovals:
    """
    Attributes:
        data (Union[Unset, List['WorkitemsListGetResponseDataItemRelationshipsApprovalsDataItem']]):
        meta (Union[Unset, WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta]):
    """

    data: Union[
        Unset,
        List["WorkitemsListGetResponseDataItemRelationshipsApprovalsDataItem"],
    ] = UNSET
    meta: Union[
        Unset, "WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.data, Unset):
            data = []
            for data_item_data in self.data:
                data_item = data_item_data.to_dict()
                data.append(data_item)

        meta: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.meta, Unset):
            meta = self.meta.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data is not UNSET:
            field_dict["data"] = data
        if meta is not UNSET:
            field_dict["meta"] = meta

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitems_list_get_response_data_item_relationships_approvals_data_item import (
            WorkitemsListGetResponseDataItemRelationshipsApprovalsDataItem,
        )
        from ..models.workitems_list_get_response_data_item_relationships_approvals_meta import (
            WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta,
        )

        d = src_dict.copy()
        data = []
        _data = d.pop("data", UNSET)
        for data_item_data in _data or []:
            data_item = WorkitemsListGetResponseDataItemRelationshipsApprovalsDataItem.from_dict(
                data_item_data
            )

            data.append(data_item)

        _meta = d.pop("meta", UNSET)
        meta: Union[
            Unset, WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta
        ]
        if isinstance(_meta, Unset):
            meta = UNSET
        else:
            meta = WorkitemsListGetResponseDataItemRelationshipsApprovalsMeta.from_dict(
                _meta
            )

        workitems_list_get_response_data_item_relationships_approvals_obj = (
            cls(
                data=data,
                meta=meta,
            )
        )

        workitems_list_get_response_data_item_relationships_approvals_obj.additional_properties = (
            d
        )
        return (
            workitems_list_get_response_data_item_relationships_approvals_obj
        )

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import json

import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_DOCUMENT_PATCH_REQUEST,
    TEST_DOCUMENT_PATCH_REQUEST2,
    TEST_DOCUMENT_POST_REQUEST,
    TEST_DOCUMENT_RESPONSE,
)


def test_get_document_with_all_fields(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_DOCUMENT_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    document = client.documents.get(
        "MySpaceId", "MyDocumentName", {"fields[documents]": "@all"}
    )

    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert isinstance(document, polarion_api.data_models.Document)
    assert len(reqs) == 1
    assert document == polarion_api.data_models.Document(
        "MyProjectId/MySpaceId/MyDocumentName",
        "MySpaceId",
        "MyDocumentName",
        "standardSpecification",
        "open",
        polarion_api.data_models.TextContent(
            type="text/html", value="<h1>My text value</h1>"
        ),
        "MyDocumentName",
        [
            polarion_api.data_models.RenderingLayout(
                type="task",
                label="My label",
                layouter=polarion_api.data_models.Layouter("paragraph"),
                properties=polarion_api.data_models.RenderingProperties(
                    fields_at_start=["id"],
                    fields_at_end=["custom", "bla"],
                    sidebar_work_item_fields=["id"],
                    fields_at_end_as_table=True,
                ),
            ),
        ],
        True,
        "PREFIX",
    )


def test_create_new_document(
    client: polarion_api.ProjectClient, httpx_mock: pytest_httpx.HTTPXMock
):
    document = polarion_api.Document(
        module_folder="folder",
        module_name="name",
        home_page_content=polarion_api.TextContent(
            type="text/html", value="<p>super Value</p>"
        ),
        title="Fancy Title",
        outline_numbering=False,
        outline_numbering_prefix="TEST",
        rendering_layouts=[
            polarion_api.data_models.RenderingLayout(
                type="task",
                label="My label",
                layouter=polarion_api.data_models.Layouter("paragraph"),
                properties=polarion_api.data_models.RenderingProperties(
                    fields_at_start=["id"],
                    fields_at_end=["custom"],
                    sidebar_work_item_fields=["id"],
                    fields_at_end_as_table=True,
                ),
            ),
            polarion_api.data_models.RenderingLayout(
                type="task2",
                label="My label",
                layouter=polarion_api.data_models.Layouter("paragraph"),
                properties=polarion_api.data_models.RenderingProperties(
                    fields_at_start=["id"],
                ),
            ),
        ],
    )

    httpx_mock.add_response(
        201,
        json={
            "data": [
                {
                    "type": "documents",
                    "id": "PROJ/folder/name",
                    "links": {
                        "self": "server-host-name/application-path/projects/PROJ/spaces/folder/documents/name?revision=1234"  # pylint: disable=line-too-long
                    },
                }
            ]
        },
    )
    client.documents.create(document)

    with open(TEST_DOCUMENT_POST_REQUEST, "r", encoding="utf-8") as f:
        expected_request = json.load(f)

    assert len(httpx_mock.get_requests()) == 1
    req = httpx_mock.get_request()
    assert req.method == "POST"
    assert (
        req.url == "http://127.0.0.1/api/projects/PROJ/spaces/folder/documents"
    )
    assert json.loads(req.content.decode("utf-8")) == expected_request


def test_update_document(
    client: polarion_api.ProjectClient, httpx_mock: pytest_httpx.HTTPXMock
):
    document = polarion_api.Document(
        module_folder="folder",
        module_name="name1",
        home_page_content=polarion_api.TextContent(
            type="text/html", value="<p>super Value</p>"
        ),
        title="Fancy Title",
    )

    document2 = polarion_api.Document(
        module_folder="folder",
        module_name="name",
        home_page_content=polarion_api.TextContent(
            type="text/html", value="<p>super Value</p>"
        ),
        title="Fancy Title",
        rendering_layouts=[
            polarion_api.data_models.RenderingLayout(
                type="task",
                label="My label",
                layouter=polarion_api.data_models.Layouter("paragraph"),
                properties=polarion_api.data_models.RenderingProperties(
                    fields_at_start=["id"],
                    fields_at_end=["custom"],
                    sidebar_work_item_fields=["id"],
                    hidden=True,
                ),
            ),
            polarion_api.data_models.RenderingLayout(
                type="task2",
                label="My label",
                layouter=polarion_api.data_models.Layouter("paragraph"),
                properties=polarion_api.data_models.RenderingProperties(
                    fields_at_start=["id"],
                ),
            ),
        ],
    )

    httpx_mock.add_response(204)
    httpx_mock.add_response(204)
    client.documents.update([document, document2])

    with open(TEST_DOCUMENT_PATCH_REQUEST, "r", encoding="utf-8") as f:
        expected_request = json.load(f)
    with open(TEST_DOCUMENT_PATCH_REQUEST2, "r", encoding="utf-8") as f:
        expected_request_2 = json.load(f)
    reqs = httpx_mock.get_requests()

    assert len(reqs) == 2
    assert reqs[0].method == "PATCH"
    assert (
        reqs[0].url
        == "http://127.0.0.1/api/projects/PROJ/spaces/folder/documents/name1"
    )
    assert json.loads(reqs[0].content.decode("utf-8")) == expected_request
    assert reqs[1].method == "PATCH"
    assert (
        reqs[1].url
        == "http://127.0.0.1/api/projects/PROJ/spaces/folder/documents/name"
    )
    assert json.loads(reqs[1].content.decode("utf-8")) == expected_request_2



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkrecordsListPostRequestDataItemAttributes")


@_attrs_define
class WorkrecordsListPostRequestDataItemAttributes:
    """
    Attributes:
        comment (Union[Unset, str]):  Example: Comment.
        date (Union[Unset, datetime.date]):  Example: 1970-01-01.
        time_spent (Union[Unset, str]):  Example: 5 1/2d.
        type (Union[Unset, str]):  Example: task.
    """

    comment: Union[Unset, str] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    time_spent: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        comment = self.comment

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        time_spent = self.time_spent

        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if comment is not UNSET:
            field_dict["comment"] = comment
        if date is not UNSET:
            field_dict["date"] = date
        if time_spent is not UNSET:
            field_dict["timeSpent"] = time_spent
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        comment = d.pop("comment", UNSET)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        time_spent = d.pop("timeSpent", UNSET)

        type = d.pop("type", UNSET)

        workrecords_list_post_request_data_item_attributes_obj = cls(
            comment=comment,
            date=date,
            time_spent=time_spent,
            type=type,
        )

        workrecords_list_post_request_data_item_attributes_obj.additional_properties = (
            d
        )
        return workrecords_list_post_request_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.projects_single_patch_request_data_attributes_description import (
        ProjectsSinglePatchRequestDataAttributesDescription,
    )


T = TypeVar("T", bound="ProjectsSinglePatchRequestDataAttributes")


@_attrs_define
class ProjectsSinglePatchRequestDataAttributes:
    """
    Attributes:
        active (Union[Unset, bool]):
        color (Union[Unset, str]):  Example: Color.
        description (Union[Unset, ProjectsSinglePatchRequestDataAttributesDescription]):
        finish (Union[Unset, datetime.date]):  Example: 1970-01-01.
        icon (Union[Unset, str]):  Example: Icon.
        lock_work_records_date (Union[Unset, datetime.date]):  Example: 1970-01-01.
        name (Union[Unset, str]):  Example: Name.
        start (Union[Unset, datetime.date]):  Example: 1970-01-01.
        tracker_prefix (Union[Unset, str]):  Example: Tracker Prefix.
    """

    active: Union[Unset, bool] = UNSET
    color: Union[Unset, str] = UNSET
    description: Union[
        Unset, "ProjectsSinglePatchRequestDataAttributesDescription"
    ] = UNSET
    finish: Union[Unset, datetime.date] = UNSET
    icon: Union[Unset, str] = UNSET
    lock_work_records_date: Union[Unset, datetime.date] = UNSET
    name: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.date] = UNSET
    tracker_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        active = self.active

        color = self.color

        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        finish: Union[Unset, str] = UNSET
        if not isinstance(self.finish, Unset):
            finish = self.finish.isoformat()

        icon = self.icon

        lock_work_records_date: Union[Unset, str] = UNSET
        if not isinstance(self.lock_work_records_date, Unset):
            lock_work_records_date = self.lock_work_records_date.isoformat()

        name = self.name

        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        tracker_prefix = self.tracker_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if active is not UNSET:
            field_dict["active"] = active
        if color is not UNSET:
            field_dict["color"] = color
        if description is not UNSET:
            field_dict["description"] = description
        if finish is not UNSET:
            field_dict["finish"] = finish
        if icon is not UNSET:
            field_dict["icon"] = icon
        if lock_work_records_date is not UNSET:
            field_dict["lockWorkRecordsDate"] = lock_work_records_date
        if name is not UNSET:
            field_dict["name"] = name
        if start is not UNSET:
            field_dict["start"] = start
        if tracker_prefix is not UNSET:
            field_dict["trackerPrefix"] = tracker_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.projects_single_patch_request_data_attributes_description import (
            ProjectsSinglePatchRequestDataAttributesDescription,
        )

        d = src_dict.copy()
        active = d.pop("active", UNSET)

        color = d.pop("color", UNSET)

        _description = d.pop("description", UNSET)
        description: Union[
            Unset, ProjectsSinglePatchRequestDataAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = (
                ProjectsSinglePatchRequestDataAttributesDescription.from_dict(
                    _description
                )
            )

        _finish = d.pop("finish", UNSET)
        finish: Union[Unset, datetime.date]
        if isinstance(_finish, Unset):
            finish = UNSET
        else:
            finish = isoparse(_finish).date()

        icon = d.pop("icon", UNSET)

        _lock_work_records_date = d.pop("lockWorkRecordsDate", UNSET)
        lock_work_records_date: Union[Unset, datetime.date]
        if isinstance(_lock_work_records_date, Unset):
            lock_work_records_date = UNSET
        else:
            lock_work_records_date = isoparse(_lock_work_records_date).date()

        name = d.pop("name", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.date]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start).date()

        tracker_prefix = d.pop("trackerPrefix", UNSET)

        projects_single_patch_request_data_attributes_obj = cls(
            active=active,
            color=color,
            description=description,
            finish=finish,
            icon=icon,
            lock_work_records_date=lock_work_records_date,
            name=name,
            start=start,
            tracker_prefix=tracker_prefix,
        )

        projects_single_patch_request_data_attributes_obj.additional_properties = (
            d
        )
        return projects_single_patch_request_data_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IconsListGetResponseDataItemAttributes")


@_attrs_define
class IconsListGetResponseDataItemAttributes:
    """
    Attributes:
        icon_url (Union[Unset, str]):  Example: pathexample.
        id (Union[Unset, str]):  Example: pathexample.
        path (Union[Unset, str]):  Example: pathexample.
    """

    icon_url: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    path: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        icon_url = self.icon_url

        id = self.id

        path = self.path

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if icon_url is not UNSET:
            field_dict["iconUrl"] = icon_url
        if id is not UNSET:
            field_dict["id"] = id
        if path is not UNSET:
            field_dict["path"] = path

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        icon_url = d.pop("iconUrl", UNSET)

        id = d.pop("id", UNSET)

        path = d.pop("path", UNSET)

        icons_list_get_response_data_item_attributes_obj = cls(
            icon_url=icon_url,
            id=id,
            path=path,
        )

        icons_list_get_response_data_item_attributes_obj.additional_properties = (
            d
        )
        return icons_list_get_response_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import copy
import json

import pytest
import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_ERROR_RESPONSE,
    TEST_WI_CREATED_RESPONSE,
    TEST_WI_DELETE_REQUEST,
    TEST_WI_ERROR_NEXT_PAGE_RESPONSE,
    TEST_WI_MULTI_POST_REQUEST,
    TEST_WI_MULTI_POST_REQUEST_IN_DOC,
    TEST_WI_NEXT_PAGE_RESPONSE,
    TEST_WI_NO_NEXT_PAGE_RESPONSE,
    TEST_WI_NOT_TRUNCATED_RESPONSE,
    TEST_WI_PATCH_COMPLETELY_REQUEST,
    TEST_WI_PATCH_DESCRIPTION_REQUEST,
    TEST_WI_PATCH_STATUS_DELETED_REQUEST,
    TEST_WI_PATCH_STATUS_REQUEST,
    TEST_WI_PATCH_TITLE_REQUEST,
    TEST_WI_POST_REQUEST,
    TEST_WI_SINGLE_RESPONSE,
)


def test_get_one_work_item(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_SINGLE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    work_item = client.work_items.get("MyWorkItemId")

    query = {
        "fields[workitems]": "@all",
        "fields[workitem_attachments]": "@all",
        "fields[linkedworkitems]": "@all",
    }
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    assert len(reqs) == 1
    assert work_item is not None
    assert len(work_item.linked_work_items) == 1
    assert len(work_item.attachments) == 1
    assert "test_custom_field" in work_item.additional_attributes
    assert work_item.attachments_truncated is True
    assert work_item.linked_work_items_truncated is True
    assert work_item.home_document is not None
    assert work_item.home_document.module_folder == "MySpaceId"
    assert work_item.home_document.module_name == "MyDocumentId"


def test_get_one_work_item_not_truncated(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_NOT_TRUNCATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    work_item = client.work_items.get("MyWorkItemId")

    query = {
        "fields[workitems]": "@all",
        "fields[workitem_attachments]": "@all",
        "fields[linkedworkitems]": "@all",
    }
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    assert len(reqs) == 1

    assert work_item is not None
    assert len(work_item.linked_work_items) == 1
    assert len(work_item.attachments) == 1
    assert "test_custom_field" in work_item.additional_attributes
    assert work_item.attachments_truncated is False
    assert work_item.linked_work_items_truncated is False


def test_get_all_work_items_multi_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))
    with open(TEST_WI_NO_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    work_items = client.work_items.get_all(
        "",
        fields={"fields[workitems]": "id"},
    )

    query = {
        "fields[workitems]": "id",
        "page[size]": "100",
        "page[number]": "1",
        "query": "",
    }
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    assert reqs[1].method == "GET"
    query["page[number]"] = "2"
    assert dict(reqs[1].url.params) == query
    assert len(work_items) == 2
    assert len(reqs) == 2
    assert work_items[0].status is None


def test_get_all_work_items_single_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_NO_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    client._client.default_fields.workitems = "@basic,description"

    work_items = client.work_items.get_all("")

    query = {
        "fields[workitems]": "@basic,description",
        "page[size]": "100",
        "page[number]": "1",
        "query": "",
    }
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert len(work_items) == 1
    assert len(reqs) == 1
    assert dict(reqs[0].url.params) == query
    assert work_items[0] == polarion_api.WorkItem(
        "MyWorkItemId2",
        title="Title",
        description=polarion_api.HtmlContent("My text value"),
        type="task",
        status="open",
        additional_attributes={"capella_uuid": "asdfgh", "checksum": "123"},
        linked_work_items=[
            polarion_api.WorkItemLink(
                "MyWorkItemId2",
                "MyLinkedWorkItemId",
                "parent",
                False,
                "MyProjectId",
            )
        ],
        attachments=[
            polarion_api.WorkItemAttachment("MyWorkItemId2", "MyAttachmentId")
        ],
    )
    assert work_items[0].home_document.module_folder == "MySpaceId"
    assert work_items[0].home_document.module_name == "MyDocumentId"
    assert "checksum" in work_items[0].additional_attributes


def test_get_all_work_items_faulty_item(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_WI_ERROR_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    with open(TEST_WI_NO_NEXT_PAGE_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    work_items = client.work_items.get_all("")
    reqs = httpx_mock.get_requests()
    assert reqs[0].method == "GET"
    assert len(work_items) == 1
    assert len(reqs) == 2


def test_create_work_item(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))

    client.work_items.create(work_item)

    req = httpx_mock.get_request()
    assert req is not None and req.method == "POST"
    with open(TEST_WI_POST_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected


def test_create_work_items_successfully(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        mock_response = json.load(f)

    mock_response["data"] *= 3
    httpx_mock.add_response(201, json=mock_response)

    client.work_items.create(3 * [work_item])

    req = httpx_mock.get_request()

    assert req is not None and req.method == "POST"
    with open(TEST_WI_MULTI_POST_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected


def test_create_work_item_in_document(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        mock_response = json.load(f)

    httpx_mock.add_response(201, json=mock_response)

    work_item.home_document = polarion_api.DocumentReference(
        "space", "document"
    )
    client.work_items.create(work_item)

    req = httpx_mock.get_request()

    assert req is not None and req.method == "POST"
    with open(TEST_WI_MULTI_POST_REQUEST_IN_DOC, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(req.content.decode()) == expected


def test_create_work_items_batch_exceed_successfully(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        mock_response = json.load(f)

    mock_response["data"] *= 3
    httpx_mock.add_response(201, json=mock_response)
    httpx_mock.add_response(201, json=mock_response)

    client.work_items.create(6 * [work_item])

    reqs = httpx_mock.get_requests()

    assert len(reqs) == 2
    assert reqs[0] is not None and reqs[0].method == "POST"
    assert reqs[1] is not None and reqs[1].method == "POST"
    with open(TEST_WI_MULTI_POST_REQUEST, encoding="utf8") as f:
        expected = json.load(f)

    assert json.loads(reqs[0].content.decode()) == expected
    assert json.loads(reqs[1].content.decode()) == expected


def test_create_work_items_slit_by_content_size_successfully(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    with open(TEST_WI_CREATED_RESPONSE, encoding="utf8") as f:
        mock_response = json.load(f)

    mock_response_data = mock_response["data"]
    mock_response["data"] = 3 * mock_response_data
    httpx_mock.add_response(201, json=mock_response)
    mock_response["data"] = 2 * mock_response_data
    httpx_mock.add_response(201, json=mock_response)
    mock_response["data"] = mock_response_data
    httpx_mock.add_response(201, json=mock_response)

    work_item_long = polarion_api.WorkItem(
        title="Title",
        description=polarion_api.HtmlContent("AB" * 512 * 1024),
        type="task",
        status="open",
        additional_attributes={"capella_uuid": "asdfg"},
    )

    work_items = [
        work_item,
        work_item_long,
        copy.deepcopy(work_item),
        copy.deepcopy(work_item_long),
        copy.deepcopy(work_item),
        copy.deepcopy(work_item_long),
    ]

    client.work_items.create(work_items)

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 3
    assert reqs[0] is not None and reqs[0].method == "POST"
    assert len(json.loads(reqs[0].content.decode("utf-8"))["data"]) == 3
    assert reqs[1] is not None and reqs[1].method == "POST"
    assert len(json.loads(reqs[1].content.decode("utf-8"))["data"]) == 2
    assert reqs[2] is not None and reqs[2].method == "POST"
    assert len(json.loads(reqs[2].content.decode("utf-8"))["data"]) == 1
    assert all(wi.id == "MyWorkItemId" for wi in work_items)
    assert all(len(req.content) <= 2 * 1024**2 for req in reqs)


def test_create_work_items_content_exceed_error(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    work_item_long = polarion_api.WorkItem(
        title="Title",
        description=polarion_api.HtmlContent("AB" * 1024 * 1024),
        type="task",
        status="open",
        additional_attributes={"capella_uuid": "asdfg"},
    )
    with pytest.raises(polarion_api.PolarionWorkItemException) as exc_info:
        client.work_items.create(3 * [work_item, work_item_long])

    assert exc_info.value.work_item == work_item_long
    assert (
        exc_info.value.args[0]
        == "A WorkItem is too large to create. (WorkItem Title: Title)"
    )
    assert len(httpx_mock.get_requests()) == 0


def test_create_work_items_failed(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    expected = (
        "Unexpected token, BEGIN_ARRAY expected, but was"
        " : BEGIN_OBJECT (at $.data)"
    )
    with open(TEST_ERROR_RESPONSE, encoding="utf8") as f:
        response = json.load(f)

    httpx_mock.add_response(400, json=response)
    httpx_mock.add_response(400, json=response)

    with pytest.raises(polarion_api.PolarionApiException) as exc_info:
        client.work_items.create(3 * [work_item])

    assert exc_info.type is polarion_api.PolarionApiException
    assert exc_info.value.args[0] == 400
    assert exc_info.value.args[1][1] == expected
    assert len(httpx_mock.get_requests()) == 2


def test_create_work_items_failed_no_error(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item: polarion_api.WorkItem,
):
    httpx_mock.add_response(501, content=b"asdfg")
    httpx_mock.add_response(501, content=b"asdfg")

    with pytest.raises(polarion_api.PolarionApiBaseException) as exc_info:
        client.work_items.create(3 * [work_item])

    assert exc_info.type is polarion_api.PolarionApiUnexpectedException
    assert exc_info.value.args[0] == 501
    assert exc_info.value.args[1] == b"asdfg"


def test_update_work_item_completely(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
    work_item_patch: polarion_api.WorkItem,
):
    httpx_mock.add_response(204)

    client.work_items.update(work_item_patch)

    req = httpx_mock.get_request()

    assert req is not None
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId")
    assert req.method == "PATCH"
    with open(TEST_WI_PATCH_COMPLETELY_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_update_work_item_description(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.update(
        polarion_api.WorkItem(
            id="MyWorkItemId",
            description=polarion_api.HtmlContent("My text value"),
        )
    )

    req = httpx_mock.get_request()
    assert req is not None
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId")
    assert req.method == "PATCH"
    with open(TEST_WI_PATCH_DESCRIPTION_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_update_work_item_title(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.update(
        polarion_api.WorkItem(id="MyWorkItemId", title="Title")
    )

    req = httpx_mock.get_request()
    assert req is not None
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId")
    assert req.method == "PATCH"
    with open(TEST_WI_PATCH_TITLE_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_update_work_item_status(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.update(
        polarion_api.WorkItem(id="MyWorkItemId", status="open")
    )

    req = httpx_mock.get_request()

    assert req is not None
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId")
    assert req.method == "PATCH"
    assert len(req.url.params) == 0
    with open(TEST_WI_PATCH_STATUS_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_update_work_item_type(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.update(
        polarion_api.WorkItem(id="MyWorkItemId", type="newType", status="open")
    )

    req = httpx_mock.get_request()
    assert req is not None
    assert req.url.path.endswith("PROJ/workitems/MyWorkItemId")
    assert req.url.params["changeTypeTo"] == "newType"
    assert req.method == "PATCH"
    with open(TEST_WI_PATCH_STATUS_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_delete_work_item_status_mode(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.delete(polarion_api.WorkItem("MyWorkItemId"))

    req = httpx_mock.get_request()
    assert req is not None and req.method == "PATCH"
    with open(TEST_WI_PATCH_STATUS_DELETED_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)


def test_delete_work_item_delete_mode(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)

    client.work_items.delete_status = None

    client.work_items.delete(polarion_api.WorkItem("MyWorkItemId"))

    req = httpx_mock.get_request()
    assert req is not None and req.method == "DELETE"
    with open(TEST_WI_DELETE_REQUEST, encoding="utf8") as f:
        assert json.loads(req.content.decode()) == json.load(f)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowActionsActionResponseBodyDataItem")


@_attrs_define
class WorkflowActionsActionResponseBodyDataItem:
    """
    Attributes:
        id (Union[Unset, str]):
        available (Union[Unset, bool]):
        cleared_fields (Union[Unset, List[str]]):
        is_adding_signature (Union[Unset, bool]):
        is_signature_required (Union[Unset, bool]):
        name (Union[Unset, str]):
        native_action_id (Union[Unset, str]):
        required_fields (Union[Unset, List[str]]):
        required_roles (Union[Unset, List[str]]):
        target_status (Union[Unset, str]):
        unavailable_reason (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    available: Union[Unset, bool] = UNSET
    cleared_fields: Union[Unset, List[str]] = UNSET
    is_adding_signature: Union[Unset, bool] = UNSET
    is_signature_required: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    native_action_id: Union[Unset, str] = UNSET
    required_fields: Union[Unset, List[str]] = UNSET
    required_roles: Union[Unset, List[str]] = UNSET
    target_status: Union[Unset, str] = UNSET
    unavailable_reason: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        available = self.available

        cleared_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.cleared_fields, Unset):
            cleared_fields = self.cleared_fields

        is_adding_signature = self.is_adding_signature

        is_signature_required = self.is_signature_required

        name = self.name

        native_action_id = self.native_action_id

        required_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.required_fields, Unset):
            required_fields = self.required_fields

        required_roles: Union[Unset, List[str]] = UNSET
        if not isinstance(self.required_roles, Unset):
            required_roles = self.required_roles

        target_status = self.target_status

        unavailable_reason = self.unavailable_reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if available is not UNSET:
            field_dict["available"] = available
        if cleared_fields is not UNSET:
            field_dict["clearedFields"] = cleared_fields
        if is_adding_signature is not UNSET:
            field_dict["isAddingSignature"] = is_adding_signature
        if is_signature_required is not UNSET:
            field_dict["isSignatureRequired"] = is_signature_required
        if name is not UNSET:
            field_dict["name"] = name
        if native_action_id is not UNSET:
            field_dict["nativeActionId"] = native_action_id
        if required_fields is not UNSET:
            field_dict["requiredFields"] = required_fields
        if required_roles is not UNSET:
            field_dict["requiredRoles"] = required_roles
        if target_status is not UNSET:
            field_dict["targetStatus"] = target_status
        if unavailable_reason is not UNSET:
            field_dict["unavailableReason"] = unavailable_reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        available = d.pop("available", UNSET)

        cleared_fields = cast(List[str], d.pop("clearedFields", UNSET))

        is_adding_signature = d.pop("isAddingSignature", UNSET)

        is_signature_required = d.pop("isSignatureRequired", UNSET)

        name = d.pop("name", UNSET)

        native_action_id = d.pop("nativeActionId", UNSET)

        required_fields = cast(List[str], d.pop("requiredFields", UNSET))

        required_roles = cast(List[str], d.pop("requiredRoles", UNSET))

        target_status = d.pop("targetStatus", UNSET)

        unavailable_reason = d.pop("unavailableReason", UNSET)

        workflow_actions_action_response_body_data_item_obj = cls(
            id=id,
            available=available,
            cleared_fields=cleared_fields,
            is_adding_signature=is_adding_signature,
            is_signature_required=is_signature_required,
            name=name,
            native_action_id=native_action_id,
            required_fields=required_fields,
            required_roles=required_roles,
            target_status=target_status,
            unavailable_reason=unavailable_reason,
        )

        workflow_actions_action_response_body_data_item_obj.additional_properties = (
            d
        )
        return workflow_actions_action_response_body_data_item_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.documents_single_post_response_data_relationships_attachments import (
        DocumentsSinglePostResponseDataRelationshipsAttachments,
    )
    from ..models.documents_single_post_response_data_relationships_author import (
        DocumentsSinglePostResponseDataRelationshipsAuthor,
    )
    from ..models.documents_single_post_response_data_relationships_branched_from import (
        DocumentsSinglePostResponseDataRelationshipsBranchedFrom,
    )
    from ..models.documents_single_post_response_data_relationships_comments import (
        DocumentsSinglePostResponseDataRelationshipsComments,
    )
    from ..models.documents_single_post_response_data_relationships_derived_from import (
        DocumentsSinglePostResponseDataRelationshipsDerivedFrom,
    )
    from ..models.documents_single_post_response_data_relationships_project import (
        DocumentsSinglePostResponseDataRelationshipsProject,
    )
    from ..models.documents_single_post_response_data_relationships_updated_by import (
        DocumentsSinglePostResponseDataRelationshipsUpdatedBy,
    )
    from ..models.documents_single_post_response_data_relationships_variant import (
        DocumentsSinglePostResponseDataRelationshipsVariant,
    )


T = TypeVar("T", bound="DocumentsSinglePostResponseDataRelationships")


@_attrs_define
class DocumentsSinglePostResponseDataRelationships:
    """
    Attributes:
        attachments (Union[Unset, DocumentsSinglePostResponseDataRelationshipsAttachments]):
        author (Union[Unset, DocumentsSinglePostResponseDataRelationshipsAuthor]):
        branched_from (Union[Unset, DocumentsSinglePostResponseDataRelationshipsBranchedFrom]):
        comments (Union[Unset, DocumentsSinglePostResponseDataRelationshipsComments]):
        derived_from (Union[Unset, DocumentsSinglePostResponseDataRelationshipsDerivedFrom]):
        project (Union[Unset, DocumentsSinglePostResponseDataRelationshipsProject]):
        updated_by (Union[Unset, DocumentsSinglePostResponseDataRelationshipsUpdatedBy]):
        variant (Union[Unset, DocumentsSinglePostResponseDataRelationshipsVariant]):
    """

    attachments: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsAttachments"
    ] = UNSET
    author: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsAuthor"
    ] = UNSET
    branched_from: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsBranchedFrom"
    ] = UNSET
    comments: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsComments"
    ] = UNSET
    derived_from: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsDerivedFrom"
    ] = UNSET
    project: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsProject"
    ] = UNSET
    updated_by: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsUpdatedBy"
    ] = UNSET
    variant: Union[
        Unset, "DocumentsSinglePostResponseDataRelationshipsVariant"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        attachments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = self.attachments.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        branched_from: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.branched_from, Unset):
            branched_from = self.branched_from.to_dict()

        comments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = self.comments.to_dict()

        derived_from: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.derived_from, Unset):
            derived_from = self.derived_from.to_dict()

        project: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        updated_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.updated_by, Unset):
            updated_by = self.updated_by.to_dict()

        variant: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.variant, Unset):
            variant = self.variant.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if author is not UNSET:
            field_dict["author"] = author
        if branched_from is not UNSET:
            field_dict["branchedFrom"] = branched_from
        if comments is not UNSET:
            field_dict["comments"] = comments
        if derived_from is not UNSET:
            field_dict["derivedFrom"] = derived_from
        if project is not UNSET:
            field_dict["project"] = project
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if variant is not UNSET:
            field_dict["variant"] = variant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.documents_single_post_response_data_relationships_attachments import (
            DocumentsSinglePostResponseDataRelationshipsAttachments,
        )
        from ..models.documents_single_post_response_data_relationships_author import (
            DocumentsSinglePostResponseDataRelationshipsAuthor,
        )
        from ..models.documents_single_post_response_data_relationships_branched_from import (
            DocumentsSinglePostResponseDataRelationshipsBranchedFrom,
        )
        from ..models.documents_single_post_response_data_relationships_comments import (
            DocumentsSinglePostResponseDataRelationshipsComments,
        )
        from ..models.documents_single_post_response_data_relationships_derived_from import (
            DocumentsSinglePostResponseDataRelationshipsDerivedFrom,
        )
        from ..models.documents_single_post_response_data_relationships_project import (
            DocumentsSinglePostResponseDataRelationshipsProject,
        )
        from ..models.documents_single_post_response_data_relationships_updated_by import (
            DocumentsSinglePostResponseDataRelationshipsUpdatedBy,
        )
        from ..models.documents_single_post_response_data_relationships_variant import (
            DocumentsSinglePostResponseDataRelationshipsVariant,
        )

        d = src_dict.copy()
        _attachments = d.pop("attachments", UNSET)
        attachments: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsAttachments
        ]
        if isinstance(_attachments, Unset):
            attachments = UNSET
        else:
            attachments = DocumentsSinglePostResponseDataRelationshipsAttachments.from_dict(
                _attachments
            )

        _author = d.pop("author", UNSET)
        author: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsAuthor
        ]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = (
                DocumentsSinglePostResponseDataRelationshipsAuthor.from_dict(
                    _author
                )
            )

        _branched_from = d.pop("branchedFrom", UNSET)
        branched_from: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsBranchedFrom
        ]
        if isinstance(_branched_from, Unset):
            branched_from = UNSET
        else:
            branched_from = DocumentsSinglePostResponseDataRelationshipsBranchedFrom.from_dict(
                _branched_from
            )

        _comments = d.pop("comments", UNSET)
        comments: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsComments
        ]
        if isinstance(_comments, Unset):
            comments = UNSET
        else:
            comments = (
                DocumentsSinglePostResponseDataRelationshipsComments.from_dict(
                    _comments
                )
            )

        _derived_from = d.pop("derivedFrom", UNSET)
        derived_from: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsDerivedFrom
        ]
        if isinstance(_derived_from, Unset):
            derived_from = UNSET
        else:
            derived_from = DocumentsSinglePostResponseDataRelationshipsDerivedFrom.from_dict(
                _derived_from
            )

        _project = d.pop("project", UNSET)
        project: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsProject
        ]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = (
                DocumentsSinglePostResponseDataRelationshipsProject.from_dict(
                    _project
                )
            )

        _updated_by = d.pop("updatedBy", UNSET)
        updated_by: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsUpdatedBy
        ]
        if isinstance(_updated_by, Unset):
            updated_by = UNSET
        else:
            updated_by = DocumentsSinglePostResponseDataRelationshipsUpdatedBy.from_dict(
                _updated_by
            )

        _variant = d.pop("variant", UNSET)
        variant: Union[
            Unset, DocumentsSinglePostResponseDataRelationshipsVariant
        ]
        if isinstance(_variant, Unset):
            variant = UNSET
        else:
            variant = (
                DocumentsSinglePostResponseDataRelationshipsVariant.from_dict(
                    _variant
                )
            )

        documents_single_post_response_data_relationships_obj = cls(
            attachments=attachments,
            author=author,
            branched_from=branched_from,
            comments=comments,
            derived_from=derived_from,
            project=project,
            updated_by=updated_by,
            variant=variant,
        )

        documents_single_post_response_data_relationships_obj.additional_properties = (
            d
        )
        return documents_single_post_response_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.plans_list_post_request_data_item_attributes_calculation_type import (
    PlansListPostRequestDataItemAttributesCalculationType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.plans_list_post_request_data_item_attributes_description import (
        PlansListPostRequestDataItemAttributesDescription,
    )
    from ..models.plans_list_post_request_data_item_attributes_home_page_content import (
        PlansListPostRequestDataItemAttributesHomePageContent,
    )


T = TypeVar("T", bound="PlansListPostRequestDataItemAttributes")


@_attrs_define
class PlansListPostRequestDataItemAttributes:
    """
    Attributes:
        allowed_types (Union[Unset, List[str]]):
        calculation_type (Union[Unset, PlansListPostRequestDataItemAttributesCalculationType]):  Example: timeBased.
        capacity (Union[Unset, float]):
        color (Union[Unset, str]):  Example: Color.
        default_estimate (Union[Unset, float]):
        description (Union[Unset, PlansListPostRequestDataItemAttributesDescription]):
        due_date (Union[Unset, datetime.date]):  Example: 1970-01-01.
        estimation_field (Union[Unset, str]):  Example: Estimation Field.
        finished_on (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        home_page_content (Union[Unset, PlansListPostRequestDataItemAttributesHomePageContent]):
        id (Union[Unset, str]):  Example: ID.
        is_template (Union[Unset, bool]):
        name (Union[Unset, str]):  Example: Name.
        previous_time_spent (Union[Unset, str]):  Example: 5 1/2d.
        prioritization_field (Union[Unset, str]):  Example: Prioritization Field.
        sort_order (Union[Unset, int]):
        start_date (Union[Unset, datetime.date]):  Example: 1970-01-01.
        started_on (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        status (Union[Unset, str]):
        use_report_from_template (Union[Unset, bool]):
    """

    allowed_types: Union[Unset, List[str]] = UNSET
    calculation_type: Union[
        Unset, PlansListPostRequestDataItemAttributesCalculationType
    ] = UNSET
    capacity: Union[Unset, float] = UNSET
    color: Union[Unset, str] = UNSET
    default_estimate: Union[Unset, float] = UNSET
    description: Union[
        Unset, "PlansListPostRequestDataItemAttributesDescription"
    ] = UNSET
    due_date: Union[Unset, datetime.date] = UNSET
    estimation_field: Union[Unset, str] = UNSET
    finished_on: Union[Unset, datetime.datetime] = UNSET
    home_page_content: Union[
        Unset, "PlansListPostRequestDataItemAttributesHomePageContent"
    ] = UNSET
    id: Union[Unset, str] = UNSET
    is_template: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    previous_time_spent: Union[Unset, str] = UNSET
    prioritization_field: Union[Unset, str] = UNSET
    sort_order: Union[Unset, int] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    started_on: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, str] = UNSET
    use_report_from_template: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        allowed_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_types, Unset):
            allowed_types = self.allowed_types

        calculation_type: Union[Unset, str] = UNSET
        if not isinstance(self.calculation_type, Unset):
            calculation_type = self.calculation_type.value

        capacity = self.capacity

        color = self.color

        default_estimate = self.default_estimate

        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        due_date: Union[Unset, str] = UNSET
        if not isinstance(self.due_date, Unset):
            due_date = self.due_date.isoformat()

        estimation_field = self.estimation_field

        finished_on: Union[Unset, str] = UNSET
        if not isinstance(self.finished_on, Unset):
            finished_on = self.finished_on.isoformat()

        home_page_content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.home_page_content, Unset):
            home_page_content = self.home_page_content.to_dict()

        id = self.id

        is_template = self.is_template

        name = self.name

        previous_time_spent = self.previous_time_spent

        prioritization_field = self.prioritization_field

        sort_order = self.sort_order

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        started_on: Union[Unset, str] = UNSET
        if not isinstance(self.started_on, Unset):
            started_on = self.started_on.isoformat()

        status = self.status

        use_report_from_template = self.use_report_from_template

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allowed_types is not UNSET:
            field_dict["allowedTypes"] = allowed_types
        if calculation_type is not UNSET:
            field_dict["calculationType"] = calculation_type
        if capacity is not UNSET:
            field_dict["capacity"] = capacity
        if color is not UNSET:
            field_dict["color"] = color
        if default_estimate is not UNSET:
            field_dict["defaultEstimate"] = default_estimate
        if description is not UNSET:
            field_dict["description"] = description
        if due_date is not UNSET:
            field_dict["dueDate"] = due_date
        if estimation_field is not UNSET:
            field_dict["estimationField"] = estimation_field
        if finished_on is not UNSET:
            field_dict["finishedOn"] = finished_on
        if home_page_content is not UNSET:
            field_dict["homePageContent"] = home_page_content
        if id is not UNSET:
            field_dict["id"] = id
        if is_template is not UNSET:
            field_dict["isTemplate"] = is_template
        if name is not UNSET:
            field_dict["name"] = name
        if previous_time_spent is not UNSET:
            field_dict["previousTimeSpent"] = previous_time_spent
        if prioritization_field is not UNSET:
            field_dict["prioritizationField"] = prioritization_field
        if sort_order is not UNSET:
            field_dict["sortOrder"] = sort_order
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if started_on is not UNSET:
            field_dict["startedOn"] = started_on
        if status is not UNSET:
            field_dict["status"] = status
        if use_report_from_template is not UNSET:
            field_dict["useReportFromTemplate"] = use_report_from_template

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.plans_list_post_request_data_item_attributes_description import (
            PlansListPostRequestDataItemAttributesDescription,
        )
        from ..models.plans_list_post_request_data_item_attributes_home_page_content import (
            PlansListPostRequestDataItemAttributesHomePageContent,
        )

        d = src_dict.copy()
        allowed_types = cast(List[str], d.pop("allowedTypes", UNSET))

        _calculation_type = d.pop("calculationType", UNSET)
        calculation_type: Union[
            Unset, PlansListPostRequestDataItemAttributesCalculationType
        ]
        if isinstance(_calculation_type, Unset):
            calculation_type = UNSET
        else:
            calculation_type = (
                PlansListPostRequestDataItemAttributesCalculationType(
                    _calculation_type
                )
            )

        capacity = d.pop("capacity", UNSET)

        color = d.pop("color", UNSET)

        default_estimate = d.pop("defaultEstimate", UNSET)

        _description = d.pop("description", UNSET)
        description: Union[
            Unset, PlansListPostRequestDataItemAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = (
                PlansListPostRequestDataItemAttributesDescription.from_dict(
                    _description
                )
            )

        _due_date = d.pop("dueDate", UNSET)
        due_date: Union[Unset, datetime.date]
        if isinstance(_due_date, Unset):
            due_date = UNSET
        else:
            due_date = isoparse(_due_date).date()

        estimation_field = d.pop("estimationField", UNSET)

        _finished_on = d.pop("finishedOn", UNSET)
        finished_on: Union[Unset, datetime.datetime]
        if isinstance(_finished_on, Unset):
            finished_on = UNSET
        else:
            finished_on = isoparse(_finished_on)

        _home_page_content = d.pop("homePageContent", UNSET)
        home_page_content: Union[
            Unset, PlansListPostRequestDataItemAttributesHomePageContent
        ]
        if isinstance(_home_page_content, Unset):
            home_page_content = UNSET
        else:
            home_page_content = PlansListPostRequestDataItemAttributesHomePageContent.from_dict(
                _home_page_content
            )

        id = d.pop("id", UNSET)

        is_template = d.pop("isTemplate", UNSET)

        name = d.pop("name", UNSET)

        previous_time_spent = d.pop("previousTimeSpent", UNSET)

        prioritization_field = d.pop("prioritizationField", UNSET)

        sort_order = d.pop("sortOrder", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _started_on = d.pop("startedOn", UNSET)
        started_on: Union[Unset, datetime.datetime]
        if isinstance(_started_on, Unset):
            started_on = UNSET
        else:
            started_on = isoparse(_started_on)

        status = d.pop("status", UNSET)

        use_report_from_template = d.pop("useReportFromTemplate", UNSET)

        plans_list_post_request_data_item_attributes_obj = cls(
            allowed_types=allowed_types,
            calculation_type=calculation_type,
            capacity=capacity,
            color=color,
            default_estimate=default_estimate,
            description=description,
            due_date=due_date,
            estimation_field=estimation_field,
            finished_on=finished_on,
            home_page_content=home_page_content,
            id=id,
            is_template=is_template,
            name=name,
            previous_time_spent=previous_time_spent,
            prioritization_field=prioritization_field,
            sort_order=sort_order,
            start_date=start_date,
            started_on=started_on,
            status=status,
            use_report_from_template=use_report_from_template,
        )

        plans_list_post_request_data_item_attributes_obj.additional_properties = (
            d
        )
        return plans_list_post_request_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitems_single_patch_request_data_relationships_assignee import (
        WorkitemsSinglePatchRequestDataRelationshipsAssignee,
    )
    from ..models.workitems_single_patch_request_data_relationships_categories import (
        WorkitemsSinglePatchRequestDataRelationshipsCategories,
    )
    from ..models.workitems_single_patch_request_data_relationships_linked_revisions import (
        WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions,
    )
    from ..models.workitems_single_patch_request_data_relationships_votes import (
        WorkitemsSinglePatchRequestDataRelationshipsVotes,
    )
    from ..models.workitems_single_patch_request_data_relationships_watches import (
        WorkitemsSinglePatchRequestDataRelationshipsWatches,
    )


T = TypeVar("T", bound="WorkitemsSinglePatchRequestDataRelationships")


@_attrs_define
class WorkitemsSinglePatchRequestDataRelationships:
    """
    Attributes:
        assignee (Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsAssignee]):
        categories (Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsCategories]):
        linked_revisions (Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions]):
        votes (Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsVotes]):
        watches (Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsWatches]):
    """

    assignee: Union[
        Unset, "WorkitemsSinglePatchRequestDataRelationshipsAssignee"
    ] = UNSET
    categories: Union[
        Unset, "WorkitemsSinglePatchRequestDataRelationshipsCategories"
    ] = UNSET
    linked_revisions: Union[
        Unset, "WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions"
    ] = UNSET
    votes: Union[
        Unset, "WorkitemsSinglePatchRequestDataRelationshipsVotes"
    ] = UNSET
    watches: Union[
        Unset, "WorkitemsSinglePatchRequestDataRelationshipsWatches"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        assignee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assignee, Unset):
            assignee = self.assignee.to_dict()

        categories: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = self.categories.to_dict()

        linked_revisions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_revisions, Unset):
            linked_revisions = self.linked_revisions.to_dict()

        votes: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.votes, Unset):
            votes = self.votes.to_dict()

        watches: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.watches, Unset):
            watches = self.watches.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if categories is not UNSET:
            field_dict["categories"] = categories
        if linked_revisions is not UNSET:
            field_dict["linkedRevisions"] = linked_revisions
        if votes is not UNSET:
            field_dict["votes"] = votes
        if watches is not UNSET:
            field_dict["watches"] = watches

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitems_single_patch_request_data_relationships_assignee import (
            WorkitemsSinglePatchRequestDataRelationshipsAssignee,
        )
        from ..models.workitems_single_patch_request_data_relationships_categories import (
            WorkitemsSinglePatchRequestDataRelationshipsCategories,
        )
        from ..models.workitems_single_patch_request_data_relationships_linked_revisions import (
            WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions,
        )
        from ..models.workitems_single_patch_request_data_relationships_votes import (
            WorkitemsSinglePatchRequestDataRelationshipsVotes,
        )
        from ..models.workitems_single_patch_request_data_relationships_watches import (
            WorkitemsSinglePatchRequestDataRelationshipsWatches,
        )

        d = src_dict.copy()
        _assignee = d.pop("assignee", UNSET)
        assignee: Union[
            Unset, WorkitemsSinglePatchRequestDataRelationshipsAssignee
        ]
        if isinstance(_assignee, Unset):
            assignee = UNSET
        else:
            assignee = (
                WorkitemsSinglePatchRequestDataRelationshipsAssignee.from_dict(
                    _assignee
                )
            )

        _categories = d.pop("categories", UNSET)
        categories: Union[
            Unset, WorkitemsSinglePatchRequestDataRelationshipsCategories
        ]
        if isinstance(_categories, Unset):
            categories = UNSET
        else:
            categories = WorkitemsSinglePatchRequestDataRelationshipsCategories.from_dict(
                _categories
            )

        _linked_revisions = d.pop("linkedRevisions", UNSET)
        linked_revisions: Union[
            Unset, WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions
        ]
        if isinstance(_linked_revisions, Unset):
            linked_revisions = UNSET
        else:
            linked_revisions = WorkitemsSinglePatchRequestDataRelationshipsLinkedRevisions.from_dict(
                _linked_revisions
            )

        _votes = d.pop("votes", UNSET)
        votes: Union[Unset, WorkitemsSinglePatchRequestDataRelationshipsVotes]
        if isinstance(_votes, Unset):
            votes = UNSET
        else:
            votes = (
                WorkitemsSinglePatchRequestDataRelationshipsVotes.from_dict(
                    _votes
                )
            )

        _watches = d.pop("watches", UNSET)
        watches: Union[
            Unset, WorkitemsSinglePatchRequestDataRelationshipsWatches
        ]
        if isinstance(_watches, Unset):
            watches = UNSET
        else:
            watches = (
                WorkitemsSinglePatchRequestDataRelationshipsWatches.from_dict(
                    _watches
                )
            )

        workitems_single_patch_request_data_relationships_obj = cls(
            assignee=assignee,
            categories=categories,
            linked_revisions=linked_revisions,
            votes=votes,
            watches=watches,
        )

        workitems_single_patch_request_data_relationships_obj.additional_properties = (
            d
        )
        return workitems_single_patch_request_data_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class TestrunsSingleGetResponseDataAttributesSelectTestCasesBy(str, Enum):
    AUTOMATEDPROCESS = "automatedProcess"
    DYNAMICLIVEDOC = "dynamicLiveDoc"
    DYNAMICQUERYRESULT = "dynamicQueryResult"
    MANUALSELECTION = "manualSelection"
    STATICLIVEDOC = "staticLiveDoc"
    STATICQUERYRESULT = "staticQueryResult"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitems_list_post_request_data_item_relationships_assignee import (
        WorkitemsListPostRequestDataItemRelationshipsAssignee,
    )
    from ..models.workitems_list_post_request_data_item_relationships_author import (
        WorkitemsListPostRequestDataItemRelationshipsAuthor,
    )
    from ..models.workitems_list_post_request_data_item_relationships_categories import (
        WorkitemsListPostRequestDataItemRelationshipsCategories,
    )
    from ..models.workitems_list_post_request_data_item_relationships_linked_revisions import (
        WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions,
    )
    from ..models.workitems_list_post_request_data_item_relationships_module import (
        WorkitemsListPostRequestDataItemRelationshipsModule,
    )


T = TypeVar("T", bound="WorkitemsListPostRequestDataItemRelationships")


@_attrs_define
class WorkitemsListPostRequestDataItemRelationships:
    """
    Attributes:
        assignee (Union[Unset, WorkitemsListPostRequestDataItemRelationshipsAssignee]):
        author (Union[Unset, WorkitemsListPostRequestDataItemRelationshipsAuthor]):
        categories (Union[Unset, WorkitemsListPostRequestDataItemRelationshipsCategories]):
        linked_revisions (Union[Unset, WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions]):
        module (Union[Unset, WorkitemsListPostRequestDataItemRelationshipsModule]):
    """

    assignee: Union[
        Unset, "WorkitemsListPostRequestDataItemRelationshipsAssignee"
    ] = UNSET
    author: Union[
        Unset, "WorkitemsListPostRequestDataItemRelationshipsAuthor"
    ] = UNSET
    categories: Union[
        Unset, "WorkitemsListPostRequestDataItemRelationshipsCategories"
    ] = UNSET
    linked_revisions: Union[
        Unset, "WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions"
    ] = UNSET
    module: Union[
        Unset, "WorkitemsListPostRequestDataItemRelationshipsModule"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        assignee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assignee, Unset):
            assignee = self.assignee.to_dict()

        author: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        categories: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = self.categories.to_dict()

        linked_revisions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_revisions, Unset):
            linked_revisions = self.linked_revisions.to_dict()

        module: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.module, Unset):
            module = self.module.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if assignee is not UNSET:
            field_dict["assignee"] = assignee
        if author is not UNSET:
            field_dict["author"] = author
        if categories is not UNSET:
            field_dict["categories"] = categories
        if linked_revisions is not UNSET:
            field_dict["linkedRevisions"] = linked_revisions
        if module is not UNSET:
            field_dict["module"] = module

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workitems_list_post_request_data_item_relationships_assignee import (
            WorkitemsListPostRequestDataItemRelationshipsAssignee,
        )
        from ..models.workitems_list_post_request_data_item_relationships_author import (
            WorkitemsListPostRequestDataItemRelationshipsAuthor,
        )
        from ..models.workitems_list_post_request_data_item_relationships_categories import (
            WorkitemsListPostRequestDataItemRelationshipsCategories,
        )
        from ..models.workitems_list_post_request_data_item_relationships_linked_revisions import (
            WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions,
        )
        from ..models.workitems_list_post_request_data_item_relationships_module import (
            WorkitemsListPostRequestDataItemRelationshipsModule,
        )

        d = src_dict.copy()
        _assignee = d.pop("assignee", UNSET)
        assignee: Union[
            Unset, WorkitemsListPostRequestDataItemRelationshipsAssignee
        ]
        if isinstance(_assignee, Unset):
            assignee = UNSET
        else:
            assignee = WorkitemsListPostRequestDataItemRelationshipsAssignee.from_dict(
                _assignee
            )

        _author = d.pop("author", UNSET)
        author: Union[
            Unset, WorkitemsListPostRequestDataItemRelationshipsAuthor
        ]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = (
                WorkitemsListPostRequestDataItemRelationshipsAuthor.from_dict(
                    _author
                )
            )

        _categories = d.pop("categories", UNSET)
        categories: Union[
            Unset, WorkitemsListPostRequestDataItemRelationshipsCategories
        ]
        if isinstance(_categories, Unset):
            categories = UNSET
        else:
            categories = WorkitemsListPostRequestDataItemRelationshipsCategories.from_dict(
                _categories
            )

        _linked_revisions = d.pop("linkedRevisions", UNSET)
        linked_revisions: Union[
            Unset, WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions
        ]
        if isinstance(_linked_revisions, Unset):
            linked_revisions = UNSET
        else:
            linked_revisions = WorkitemsListPostRequestDataItemRelationshipsLinkedRevisions.from_dict(
                _linked_revisions
            )

        _module = d.pop("module", UNSET)
        module: Union[
            Unset, WorkitemsListPostRequestDataItemRelationshipsModule
        ]
        if isinstance(_module, Unset):
            module = UNSET
        else:
            module = (
                WorkitemsListPostRequestDataItemRelationshipsModule.from_dict(
                    _module
                )
            )

        workitems_list_post_request_data_item_relationships_obj = cls(
            assignee=assignee,
            author=author,
            categories=categories,
            linked_revisions=linked_revisions,
            module=module,
        )

        workitems_list_post_request_data_item_relationships_obj.additional_properties = (
            d
        )
        return workitems_list_post_request_data_item_relationships_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import json
from io import BytesIO
from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, File, FileJsonType, Unset

if TYPE_CHECKING:
    from ..models.teststepresult_attachments_list_post_request import (
        TeststepresultAttachmentsListPostRequest,
    )


T = TypeVar("T", bound="PostTestStepResultAttachmentsRequestBody")


@_attrs_define
class PostTestStepResultAttachmentsRequestBody:
    """
    Attributes:
        files (Union[Unset, List[File]]):
        resource (Union[Unset, TeststepresultAttachmentsListPostRequest]):
    """

    files: Union[Unset, List[File]] = UNSET
    resource: Union[Unset, "TeststepresultAttachmentsListPostRequest"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        files: Union[Unset, List[FileJsonType]] = UNSET
        if not isinstance(self.files, Unset):
            files = []
            for files_item_data in self.files:
                files_item = files_item_data.to_tuple()

                files.append(files_item)

        resource: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.resource, Unset):
            resource = self.resource.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if files is not UNSET:
            field_dict["files"] = files
        if resource is not UNSET:
            field_dict["resource"] = resource

        return field_dict

    def to_multipart(self) -> List[Tuple[str, Any]]:
        field_list: List[Tuple[str, Any]] = []
        for cont in self.files or []:
            files_item = cont.to_tuple()

            field_list.append(("files", files_item))

        resource: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.resource, Unset):
            resource = (
                None,
                json.dumps(self.resource.to_dict()).encode(),
                "text/plain",
            )

        if resource is not UNSET:
            field_list.append(("resource", resource))

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                key: (None, str(value).encode(), "text/plain")
                for key, value in self.additional_properties.items()
            }
        )

        field_list += list(field_dict.items())

        return field_list

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.teststepresult_attachments_list_post_request import (
            TeststepresultAttachmentsListPostRequest,
        )

        d = src_dict.copy()
        files = []
        _files = d.pop("files", UNSET)
        for files_item_data in _files or []:
            files_item = File(payload=BytesIO(files_item_data))

            files.append(files_item)

        _resource = d.pop("resource", UNSET)
        resource: Union[Unset, TeststepresultAttachmentsListPostRequest]
        if isinstance(_resource, Unset):
            resource = UNSET
        else:
            resource = TeststepresultAttachmentsListPostRequest.from_dict(
                _resource
            )

        post_test_step_result_attachments_request_body_obj = cls(
            files=files,
            resource=resource,
        )

        post_test_step_result_attachments_request_body_obj.additional_properties = (
            d
        )
        return post_test_step_result_attachments_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
import json

import pytest_httpx

import polarion_rest_api_client as polarion_api
from tests.conftest import (
    TEST_TRUN_CREATED_RESPONSE,
    TEST_TRUN_FULLY_PATCH_REQUEST,
    TEST_TRUN_NEXT_RESPONSE,
    TEST_TRUN_NO_NEXT_RESPONSE,
    TEST_TRUN_PATCH_REQUEST,
    TEST_TRUN_POST_REQUEST,
)


def test_get_test_runs_multi_page(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_TRUN_NEXT_RESPONSE, encoding="utf8") as f:
        content = json.load(f)
        httpx_mock.add_response(json=content)
        httpx_mock.add_response(json=content)

    with open(TEST_TRUN_NO_NEXT_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(json=json.load(f))

    test_runs = client.test_runs.get_all("123", fields={"test_runs": "@all"})

    query = {
        "page[size]": "100",
        "page[number]": "1",
        "fields[test_runs]": "@all",
        "query": "123",
    }
    reqs = httpx_mock.get_requests()
    assert len(reqs) == 3
    assert reqs[0].method == "GET"
    assert dict(reqs[0].url.params) == query
    query["page[number]"] = "2"
    assert dict(reqs[1].url.params) == query
    query["page[number]"] = "3"
    assert dict(reqs[2].url.params) == query
    assert len(test_runs) == 3
    assert test_runs[0].id == "MyTestRunId2"
    assert test_runs[0].type == "manual"
    assert (
        test_runs[0].select_test_cases_by
        == polarion_api.SelectTestCasesBy.MANUALSELECTION
    )
    assert test_runs[0].home_page_content
    assert test_runs[0].home_page_content.value == "My text value"
    assert test_runs[0].home_page_content.type == "text/html"
    assert test_runs[0].status == "open"
    assert test_runs[0].title == "Title"


def test_create_test_runs(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    with open(TEST_TRUN_CREATED_RESPONSE, encoding="utf8") as f:
        httpx_mock.add_response(201, json=json.load(f))

    tr_1 = polarion_api.TestRun(
        "ID",
        "manual",
        "open",
        "Title",
        polarion_api.TextContent("text/html", "My text value"),
        datetime.datetime.utcfromtimestamp(0),
        "Group ID",
        "ID Prefix",
        True,
        True,
        "Query",
        True,
        polarion_api.SelectTestCasesBy.MANUALSELECTION,
        {},
    )
    tr_2 = polarion_api.TestRun(
        "ID2",
        "manual",
        "open",
        "Title",
        polarion_api.TextContent("text/html", "My text value 2"),
        datetime.datetime.utcfromtimestamp(0),
        "Group ID",
        "ID Prefix",
        True,
        True,
        "Query",
        True,
        polarion_api.SelectTestCasesBy.MANUALSELECTION,
        {},
    )

    client.test_runs.create([tr_1, tr_2])

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 1
    req_data = json.loads(reqs[0].content.decode("utf-8"))
    with open(TEST_TRUN_POST_REQUEST, encoding="utf8") as f:
        expected_req = json.load(f)

    assert req_data == expected_req
    assert tr_1.id == "MyTestRunId"
    assert tr_2.id == "MyTestRunId2"


def test_update_test_run(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)
    test_run_id = "asdfg"
    tr = polarion_api.TestRun(
        test_run_id,
        "manual",
        "passed",
        "Title",
        finished_on=datetime.datetime.utcfromtimestamp(0),
        query="Query",
        use_report_from_template=False,
    )

    client.test_runs.update(tr)

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 1
    req_data = json.loads(reqs[0].content.decode("utf-8"))
    with open(TEST_TRUN_PATCH_REQUEST, encoding="utf8") as f:
        expected_req = json.load(f)

    assert req_data == expected_req
    assert reqs[0].url.path.endswith(f"/testruns/{test_run_id}")


def test_update_test_run_fully(
    client: polarion_api.ProjectClient,
    httpx_mock: pytest_httpx.HTTPXMock,
):
    httpx_mock.add_response(204)
    tr = polarion_api.TestRun(
        "ID",
        "manual",
        "open",
        "Title",
        polarion_api.TextContent("text/html", "My text value"),
        datetime.datetime.utcfromtimestamp(0),
        "Group ID",
        "ID Prefix",
        True,
        True,
        "Query",
        True,
        polarion_api.SelectTestCasesBy.MANUALSELECTION,
        {},
    )

    client.test_runs.update(tr)

    reqs = httpx_mock.get_requests()
    assert len(reqs) == 1
    req_data = json.loads(reqs[0].content.decode("utf-8"))
    with open(TEST_TRUN_FULLY_PATCH_REQUEST, encoding="utf8") as f:
        expected_req = json.load(f)

    assert req_data == expected_req



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LinkedworkitemsListPostRequestDataItemAttributes")


@_attrs_define
class LinkedworkitemsListPostRequestDataItemAttributes:
    """
    Attributes:
        revision (Union[Unset, str]):  Example: 1234.
        role (Union[Unset, str]):  Example: relates_to.
        suspect (Union[Unset, bool]):
    """

    revision: Union[Unset, str] = UNSET
    role: Union[Unset, str] = UNSET
    suspect: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        revision = self.revision

        role = self.role

        suspect = self.suspect

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if revision is not UNSET:
            field_dict["revision"] = revision
        if role is not UNSET:
            field_dict["role"] = role
        if suspect is not UNSET:
            field_dict["suspect"] = suspect

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        revision = d.pop("revision", UNSET)

        role = d.pop("role", UNSET)

        suspect = d.pop("suspect", UNSET)

        linkedworkitems_list_post_request_data_item_attributes_obj = cls(
            revision=revision,
            role=role,
            suspect=suspect,
        )

        linkedworkitems_list_post_request_data_item_attributes_obj.additional_properties = (
            d
        )
        return linkedworkitems_list_post_request_data_item_attributes_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.testrecords_list_get_response_data_item_attributes_comment_type import (
    TestrecordsListGetResponseDataItemAttributesCommentType,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="TestrecordsListGetResponseDataItemAttributesComment")


@_attrs_define
class TestrecordsListGetResponseDataItemAttributesComment:
    """
    Attributes:
        type (Union[Unset, TestrecordsListGetResponseDataItemAttributesCommentType]):
        value (Union[Unset, str]):  Example: My text value.
    """

    type: Union[
        Unset, TestrecordsListGetResponseDataItemAttributesCommentType
    ] = UNSET
    value: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[
            Unset, TestrecordsListGetResponseDataItemAttributesCommentType
        ]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = TestrecordsListGetResponseDataItemAttributesCommentType(
                _type
            )

        value = d.pop("value", UNSET)

        testrecords_list_get_response_data_item_attributes_comment_obj = cls(
            type=type,
            value=value,
        )

        testrecords_list_get_response_data_item_attributes_comment_obj.additional_properties = (
            d
        )
        return testrecords_list_get_response_data_item_attributes_comment_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Polarion API related errors."""
from __future__ import annotations

from . import data_models as dm

__all__ = [
    "PolarionApiBaseException",
    "PolarionApiException",
    "PolarionApiInternalException",
    "PolarionApiUnexpectedException",
    "PolarionWorkItemException",
]


class PolarionApiBaseException(Exception):
    """Base exception, which is raised, if an API error occurs."""


class PolarionApiInternalException(Exception):
    """Exception being raised, if an error occurs in the client itself."""


class PolarionWorkItemException(PolarionApiInternalException):
    """Exception being raised, if a WorkItem related error occurs."""

    def __init__(self, message: str, work_item: dm.WorkItem):
        self.work_item = work_item
        message = f"{message} (WorkItem Title: {work_item.title})"
        super().__init__(message)


class PolarionApiException(PolarionApiBaseException):
    """Exception, which is raised, if an error is raised by the API."""


class PolarionApiUnexpectedException(PolarionApiBaseException):
    """Exception, which is raised, if an unexpected error is raised."""



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from io import BytesIO
from typing import Any, Dict, List, Tuple, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, File, FileJsonType, Unset

T = TypeVar("T", bound="UpdateAvatarRequestBody")


@_attrs_define
class UpdateAvatarRequestBody:
    """
    Attributes:
        content (Union[Unset, File]): avatar content
    """

    content: Union[Unset, File] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        content: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.content, Unset):
            content = self.content.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    def to_multipart(self) -> List[Tuple[str, Any]]:
        field_list: List[Tuple[str, Any]] = []
        content: Union[Unset, FileJsonType] = UNSET
        if not isinstance(self.content, Unset):
            content = self.content.to_tuple()

        if content is not UNSET:
            field_list.append(("content", content))

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                key: (None, str(value).encode(), "text/plain")
                for key, value in self.additional_properties.items()
            }
        )

        field_list += list(field_dict.items())

        return field_list

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _content = d.pop("content", UNSET)
        content: Union[Unset, File]
        if isinstance(_content, Unset):
            content = UNSET
        else:
            content = File(payload=BytesIO(_content))

        update_avatar_request_body_obj = cls(
            content=content,
        )

        update_avatar_request_body_obj.additional_properties = d
        return update_avatar_request_body_obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Contains shared errors types that can be raised from API functions."""


class UnexpectedStatus(Exception):
    """Raised by api functions when the response status an undocumented status
    and Client.raise_on_unexpected_status is True."""

    def __init__(self, status_code: int, content: bytes):
        self.status_code = status_code
        self.content = content

        super().__init__(f"Unexpected status code: {status_code}")


__all__ = ["UnexpectedStatus"]



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client.api.test_runs import (
    get_test_runs,
    patch_test_run,
    post_test_runs,
)

from . import base_classes as bc
from . import test_records

if t.TYPE_CHECKING:
    from polarion_rest_api_client import client as polarion_client


class TestRuns(
    bc.SingleUpdatableItemsMixin[dm.TestRun],
    bc.UpdatableItemsClient[dm.TestRun],
):
    def get(self, *args, **kwargs) -> dm.TestRun:
        raise NotImplementedError

    def __init__(
        self, project_id: str, client: "polarion_client.PolarionClient"
    ):
        super().__init__(project_id, client)
        self.records = test_records.TestRecords(project_id, client)

    def _update(self, to_update: list[dm.TestRun] | dm.TestRun):
        """Create the given list of test runs."""
        assert not isinstance(to_update, list), "Expected only one item"
        assert to_update.id
        response = patch_test_run.sync_detailed(
            self._project_id,
            to_update.id,
            client=self._client.client,
            body=api_models.TestrunsSinglePatchRequest(
                data=api_models.TestrunsSinglePatchRequestData(
                    type=api_models.TestrunsSinglePatchRequestDataType.TESTRUNS,  # pylint: disable=line-too-long
                    id=f"{self._project_id}/{to_update.id}",
                    attributes=self._fill_test_run_attributes(
                        api_models.TestrunsSinglePatchRequestDataAttributes,
                        to_update,
                    ),
                )
            ),
        )

        self._raise_on_error(response)

    def get_multi(  # type: ignore[override]
        self,
        query: str = "",
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: t.Optional[dict[str, str]] = None,
    ) -> tuple[list[dm.TestRun], bool]:
        """Return the test runs on a defined page matching the given query.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        if fields is None:
            fields = self._client.default_fields.testruns

        sparse_fields = self._build_sparse_fields(fields)
        response = get_test_runs.sync_detailed(
            self._project_id,
            client=self._client.client,
            query=query,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        self._raise_on_error(response)

        parsed_response = response.parsed
        assert isinstance(parsed_response, api_models.TestrunsListGetResponse)

        test_runs = []
        for data in parsed_response.data or []:
            assert isinstance(data.id, str)
            assert isinstance(
                data.attributes,
                api_models.TestrunsListGetResponseDataItemAttributes,
            )
            test_runs.append(
                dm.TestRun(
                    data.id.split("/")[-1],
                    self.unset_to_none(data.attributes.type),
                    self.unset_to_none(data.attributes.status),
                    self.unset_to_none(data.attributes.title),
                    self._handle_text_content(
                        data.attributes.home_page_content
                    ),
                    self.unset_to_none(data.attributes.finished_on),
                    self.unset_to_none(data.attributes.group_id),
                    self.unset_to_none(data.attributes.id_prefix),
                    self.unset_to_none(data.attributes.is_template),
                    self.unset_to_none(data.attributes.keep_in_history),
                    self.unset_to_none(data.attributes.query),
                    self.unset_to_none(data.attributes.keep_in_history),
                    (
                        dm.SelectTestCasesBy(
                            str(data.attributes.select_test_cases_by)
                        )
                        if data.attributes.select_test_cases_by
                        else None
                    ),
                    data.attributes.additional_properties or {},
                )
            )
        next_page = isinstance(
            parsed_response.links,
            api_models.TestrunsListGetResponseLinks,
        ) and bool(parsed_response.links.next_)

        return test_runs, next_page

    def _create(self, items: list[dm.TestRun]):
        """Create the given list of test runs."""
        polarion_test_runs = [
            api_models.TestrunsListPostRequestDataItem(
                type=api_models.TestrunsListPostRequestDataItemType.TESTRUNS,
                attributes=self._fill_test_run_attributes(
                    api_models.TestrunsListPostRequestDataItemAttributes,
                    test_run,
                ),
            )
            for test_run in items
        ]

        response = post_test_runs.sync_detailed(
            self._project_id,
            client=self._client.client,
            body=api_models.TestrunsListPostRequest(polarion_test_runs),
        )

        self._raise_on_error(response)
        assert (
            isinstance(response.parsed, api_models.TestrunsListPostResponse)
            and response.parsed.data
        )
        if response.parsed and response.parsed.data:
            for i, data in enumerate(response.parsed.data):
                assert data.id
                items[i].id = data.id.split("/")[-1]

    def _delete(self, items: dm.TestRun | list[dm.TestRun]):
        raise NotImplementedError

    def _fill_test_run_attributes(
        self,
        attributes_type: type[
            api_models.TestrunsListPostRequestDataItemAttributes
            | api_models.TestrunsSinglePatchRequestDataAttributes
        ],
        test_run: dm.TestRun,
    ):
        type_prefix = attributes_type.__name__
        attributes = attributes_type()
        if test_run.type is not None:
            attributes.type = test_run.type
        if test_run.id and hasattr(attributes, "id"):
            attributes.id = test_run.id
        if test_run.status is not None:
            attributes.status = test_run.status
        if test_run.title is not None:
            attributes.title = test_run.title
        if test_run.finished_on is not None:
            attributes.finished_on = test_run.finished_on
        if test_run.group_id is not None:
            attributes.group_id = test_run.group_id
        if test_run.id_prefix is not None:
            attributes.id_prefix = test_run.id_prefix
        if test_run.is_template is not None and hasattr(
            attributes, "is_template"
        ):
            attributes.is_template = test_run.is_template
        if test_run.keep_in_history is not None:
            attributes.keep_in_history = test_run.keep_in_history
        if test_run.query is not None:
            attributes.query = test_run.query
        if test_run.use_report_from_template is not None:
            attributes.use_report_from_template = (
                test_run.use_report_from_template
            )
        if test_run.additional_attributes:
            attributes.additional_properties = test_run.additional_attributes
        if test_run.select_test_cases_by:
            attributes.select_test_cases_by = getattr(
                api_models, f"{type_prefix}SelectTestCasesBy"
            )(test_run.select_test_cases_by.value)
        if test_run.home_page_content:
            attributes.home_page_content = getattr(
                api_models, f"{type_prefix}HomePageContent"
            )()
            assert attributes.home_page_content
            if test_run.home_page_content.type:
                attributes.home_page_content.type = getattr(
                    api_models, f"{type_prefix}HomePageContentType"
                )(test_run.home_page_content.type)
            if test_run.home_page_content.value:
                attributes.home_page_content.value = (
                    test_run.home_page_content.value
                )

        return attributes
